@@ -138,7 +138,6 @@ comics = {
 		"wulff":{"name":"Wulffmorgenthaler","link":"http://wulffmorgenthaler.com/","start":"edd3411b-96ca-4d93-bd5f-0cf1deb67c8a","dbfile":dbdir + "comicdb.wulff.csv"},
 		"xkcd":{"name":"xkcd","link":"http://xkcd.org/","start":666,"dbfile":dbdir + "comicdb.xkcd.csv"},
 		"zack_hillcomicscom":{"name":"Zack Hill","link":'http://comics.com/zack_hill/',"start":"2010-02-01","dbfile":dbdir + "comicdb.zackhill.csv"},
-
 		"amazing_spidermanckdm":{"name":"The Amazing Spider-Man","link":'http://content.comicskingdom.net/Spiderman/',"start":"20100209","dbfile":dbdir + "comicdb.spiderman.csv"},
 		"apartment_3-gckdm":{"name":"Apartment 3-G","link":'http://content.comicskingdom.net/Apartment_3-G/',"start":"20100209","dbfile":dbdir + "comicdb.partment3g.csv"},
 		"arcticckdm":{"name":"Arctic Circle","link":'http://content.comicskingdom.net/Arctic/',"start":"20100209","dbfile":dbdir + "comicdb.arctic.csv"},
@@ -269,6 +268,29 @@ class ComicDb():
 				print "comic db creation failed after copy"
 
 
+	def get_stored(self):
+		try:
+			idf = open(self.dbfile + ".last", 'r')
+			retval = idf.read()
+			print "got stored index :" + str(retval)
+			idf.close()
+			return int(retval)
+		except:
+			return 0
+
+	def store_last(self, sid = 0):
+		try:
+			idf = open(self.dbfile + ".last", 'w')
+			if sid > 0:
+				idf.write(str(sid))
+			else:
+				idf.write(str(self.currentcomic))
+			idf.close()
+			return True
+		except:			
+			return False
+
+
 	def get_comic(self):
 		# print str(self.currentcomic)
 		if self.currentcomic < 0:
@@ -343,6 +365,9 @@ class ComicDb():
 		dbw = csv.writer(dbf)
 		dbw.writerow(irow)
 		dbf.close()
+		stored = self.get_stored()
+		if stored > 0:
+			self.store_last(stored + 1)
 
 	def insert_row_first(self, irow):
 		dbf = open(self.dbfile, 'w')
@@ -1109,7 +1134,7 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 
 	# **************** Cairo draw functions ***************
 
-	def realize(self, widget):
+	def do_realize(self):
 		global supports_alpha
 
 		# To check if the display supports alpha channels, get the colormap
@@ -1126,7 +1151,6 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 		# Now we have a colormap appropriate for the screen, use it
 		self.set_colormap(colormap)
 		hildondesktop.HomePluginItem.do_realize(self)
-		return False
 
 	def do_expose_event(widget, event):
 		cr = widget.window.cairo_create()
@@ -1146,7 +1170,7 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 			cr.set_source_rgb (0.0, 0.0, 0.0)
 			
 		cr.fill_preserve()
-		return False
+		hildondesktop.HomePluginItem.do_expose_event(widget,event)
 
 	# Function that draws a button preloaded in the images dictionary.
 	def draw(self, target, postfix):
@@ -1213,8 +1237,16 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 				self.e_goweb.show_all()
 
 			elif func == 'switch':
+				# Check if store needed..
+				if not self.db.currentcomic == 0:
+					print "Storing location " + str(self.db.currentcomic)
+					if self.db.store_last():
+						print "stored successfully"
+					else:
+						print "store failed"
+				 
 				self.active_comics = self.get_active_comics()
-				print "closing log"
+				# print "closing log"
 				#sys.stdout.close()
 				#sys.stdout = sys.__stdout__
 				#sys.stderr.close()
@@ -1284,12 +1316,14 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
   		scaled_buf = pixbuf.scale_simple(new_width,new_height,gtk.gdk.INTERP_BILINEAR)
 		print "scaled height: " + str(scaled_buf.get_height())
 		if scaled_buf.get_height() > maxheight:
+			self.scrollable = True
 			if scaled_buf.get_height() - (vpos + maxheight) < 0:
 				vpos = scaled_buf.get_height() - maxheight
 			scaled_buf.copy_area(0,vpos,picwidth,maxheight,parent_buf,1,0)
 			# parent_buf.composite(scaled_buf, 0, 0, 478, maxheight, 0, 0, 478, maxheight, gtk.gdk.INTERP_BILINEAR, 0)
 			self.imgvpos = vpos
 		else:
+			self.scrollable = False
 			tmpy = maxheight - scaled_buf.get_height()
 			if tmpy > 0:
 				tmpy = tmpy / 2
@@ -1371,6 +1405,20 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 
 		#if not, write defaults
 
+	def goto_comic(self, index):
+		self.db.currentcomic = index
+		self.e_open.remove(self.comic_image)
+		self.comic_image = self.get_resized_pixmap(self.db.get_comic(), self.imgvpos)
+		self.e_open.add(self.comic_image)
+		self.e_open.show_all()
+		self.e_goweb.remove(self.label)
+		titl = str(self.db.db[self.db.currentcomic]['title'])
+		if len(titl) > 22:
+			titl = titl[:19] + "..."
+		self.label.set_markup(self.get_markup())
+		self.e_goweb.add(self.label)
+		self.e_goweb.show_all()
+	
 # ************************* OPTION DIALOGS ********************************
 
 	def show_options(self, widget):
@@ -1387,12 +1435,32 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 		comics_button.set_alignment(0,0,0,0)
 		comics_button.connect("clicked", self.show_comics)
 
+		navfirst_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+		navfirst_button.set_text("Go to earliest", "Jump to the oldest cached comic")
+		navfirst_button.set_alignment(0,0,0,0)
+		navfirst_button.connect("clicked", self.show_nav, "oldest")
+
+		navlast_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+		navlast_button.set_text("Go to latest", "Jump to the newest cached comic")
+		navlast_button.set_alignment(0,0,0,0)
+		navlast_button.connect("clicked", self.show_nav, "newest")
+
+		navstored_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+		navstored_button.set_text("Go to stored comic", "Jump to where you were before you switched comics")
+		navstored_button.set_alignment(0,0,0,0)
+		navstored_button.connect("clicked", self.show_nav, "stored")
+
 		# size_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
 		# size_button.set_text("Change size", "Change the size of the widget")
 		# size_button.set_alignment(0,0,0,0)
 		# size_button.connect("clicked", self.show_size)
 
+
+
 		print "adding buttons to dialog, starting with comics"
+		dialog.vbox.pack_start(navstored_button, True, True, 0)
+		dialog.vbox.pack_start(navfirst_button, True, True, 0)
+		dialog.vbox.pack_start(navlast_button, True, True, 0)
 		dialog.vbox.pack_start(comics_button, True, True, 0)
 		print "adding about button"
 		dialog.vbox.pack_start(about_button, True, True, 0)
@@ -1402,6 +1470,14 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 		dialog.run()
 		dialog.destroy()
 
+	def show_nav(self, widget, goto):
+		if goto == "stored":
+			print "look for stored file"
+			self.goto_comic(self.db.get_stored())
+		elif goto == "newest":
+			self.goto_comic(0)
+		elif goto == "oldest":
+			self.goto_comic(len(self.db.db) - 1)
 	def show_about(self, widget):
 		print "in about dialog"
 		dialog = gtk.AboutDialog()
